/*
 * Project name:
     Boot_Test (using mikroC RS232-Bootloader on PIC18's 96K ROM)
 * Copyright:
     (c) FOURNET Olivier 06/08/2017
 * Revision History:
     20051120:
       - initial release (for 64K ROM)
         20170805 :
           - adaptation du code initial de Mikroelektronika  une version PIC18F  ROM 96K
         20170806 :
           - simplification du code source
 * Description:
     This is a serial port Bootloader implementation. All required bootloader
     routines are placed in the Bootloader file, the only thing that is
     user-configurable is the baudrate of the UART communication, which is set
     in the main().
     Once started in PIC, the bootloader waits for a while (approx. 5 seconds)
     to establish UART communication with the MikroBootloader application on the
     PC. If it fails to do so, the bootloader starts the programme already
     loaded in the MCU (initially it is just a NOP). If the communication is
     established, the bootloader receives the hex file from the PC and places it
     where requred (the MikroBootloader application reads out the hex file and
     takes care of the location where hex is to be placed). When hex download
     has finished, the user is notified by the MikroBootloader to reset the MCU,
     and MCU enters the described bootload sequence again.
 * Test configuration:
     MCU:             PIC18F4685
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39761c.pdf
     Oscillator:      HS, 12.0000 MHz
     Ext. Modules:    -
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
     - It's recommended not to alter the start address of the main().
       Otherwise, less space in ROM will be available for the application being bootloaded.
     - RX and TX UART switches should be turned On (board specific).
 */
#include "built_in.h"

#define SIZE_ROM                     98304         //        (0x18000)
//  0x40 = 64
#define START_PROGRAM_ADDR           SIZE_ROM - 64 // 98240  (0x17FC0)  Start_Program()
#define START_BOOTLOAD_ADRR          97996         //        (0x17ECC)  Start_Bootload()
#define START_MAIN_BOOTLOAD_ADRR     97656         //        (0x17D78)  main()
#pragma orgall                       97632         //        (0x17D60)

void Start_Bootload();
void Start_Program();

unsigned short UART_Write_Loop(char send, char receive);

/*
 * constants :
 *  _FLASH_WRITE_LATCH - flash write latch size
 *  _FLASH_ERASE       - flash erase block size
 * are related to mcu flash memory write and erase block sizes
 * and as such are defined in appropriate flash library file.
 * Be aware that these constants are MCU dependent.
 * Compiler will link them from appropriate flash library,
 * so the user does not have to take care of them.
 *
 */
static char block[64];

//-------------- Empty function, this is where the start address of 'real'
//               program will be placed (by Write_Begin()). That is why
//               this function must have at least 4 bytes of ROM available
//               after its org address.
// _Start_Program 2 Bytes
void Start_Program() org START_PROGRAM_ADDR {
}

unsigned short UART_Write_Loop(char send, char receive)
{
  unsigned short rslt = 0;
  
  while(1)
  {
    Delay_5ms();
    UART1_Write(send);
    Delay_5ms();
    
    rslt++;

  if(rslt == 255u)  return 0;
  if(UART1_Read() == receive) return 1;
  }
}

//-------------- This void will recover the bootlocation 0x0000, 0x0001 and
//               0x0002 to point to the bootloaer's main. It will also move
//               the reset vector of the program that is uploaded to a new
//               location, in this case START_MAIN_BOOTLOAD_ADRR

void Write_Begin()
{
  FLASH_Erase_Write_64(START_PROGRAM_ADDR, block);
  //--- goto main START_MAIN_BOOTLOAD_ADRR
  block[0] = 0xBC;
  block[1] = 0xEF;
  block[2] = 0xBE;
  block[3] = 0xF0;
}

//-------------- Starts with bootload
void Start_Bootload() org START_BOOTLOAD_ADRR {
  unsigned short i = 0, xx, yy;
  long j = 0;

  while(1) 
  {
    if(i == 64u) 
    {
      //--- If 32 words (64 bytes) recieved then write to flash
      if(!j) Write_Begin();
      
      //if(j<START_MAIN_BOOTLOAD_ADRR)
          FLASH_Erase_Write_64(j, block);
      i = 0;
      j += 0x40;
    }
    //--- Ask for yy
    UART1_Write('y');
    while(!UART1_Data_Ready()) ;
    //--- Read yy
    yy = UART1_Read();
    //--- Ask for xx
    UART1_Write('x');
    while(!UART1_Data_Ready()) ;
    //--- Read xx
    xx = UART1_Read();
    //--- Save xxyy in block[i]
    block[i++] = yy;
    block[i++] = xx;
  }
}

//-------------- Boootloader main for 96K PIC18's
void main() org START_MAIN_BOOTLOAD_ADRR {

  UART1_Init(9600);                 // Init USART at 9600

  if(UART_Write_Loop('g','r')) {     // Send 'g' for ~5 sec, if 'r'
    Start_Bootload();                 //   received start bootload
  }
  else
  {
   Start_Program();                  //   else start program
  }
}